#include "Global.h"
#include "Player.h"
#include <linklist.h>
#include <allegro.h>
#include <jrnd.h>
#include "Sounds.h"

volatile int fps = 0;
void fpsproc() {
  fps++;
} END_OF_FUNCTION(fpsproc);

bool WaitForAck(); //false if ESC, true if ENTER or joy
void DrawField();
void InitPlayers(LinkList<Player>& players);
void DoCraters();
void PrintResults(LinkList<Player>& players, int dead);

int craterdelay;

void PlayGame(LinkList<Player>& players) {
  LOCK_FUNCTION(fpsproc);
  LOCK_VARIABLE(fps);
  acquire_screen();
  clear(screen);
  textprintf_centre(screen, font, scrx/2, scry/2, makecol(255, 255, 255), "Press ENTER/trigger to start or ESC");
  release_screen();

  craterdelay = crater_rate;
    
  while (WaitForAck()) {
    DrawField();
    InitPlayers(players);

    //Main Game Loop
    int l = players.Length();
    int dead = 0;
    fps = 0;
    install_int(fpsproc, game_speed);
    while ((l - dead > 1) && !key[KEY_ESC]) {
      if (num_joysticks)
        poll_joystick();

      //Create craters
      if (craters) {
        DoCraters();
      }

      players.SetToStart();
      int l = players.Length();
      for (int c=0; c<l; c++) {
        Player* temp = players._GetNext();
        if (!temp->IsDead()) {
          temp->Update();
          if (!temp->Calc()) {
            temp->Draw();
          } else
            dead++;
        }
        players._Advance();
      }

      while (!fps) {}
      fps--;
    }
  
    PrintResults(players, dead);
  }
  remove_int(fpsproc);
}

bool WaitForAck() { //false if ESC, true if ENTER or joy
  bool ret = false;
  bool exit = false;
  while (!exit) {
    if (key[KEY_ENTER])
      ret = exit = true;
    if (key[KEY_ESC]) {
      exit = true;
      ret = false;
    }
    poll_joystick();
    for (int c = 0; c<num_joysticks; c++) {
      if (joy[c].button[0].b)
        ret = exit = true;
    }
  }
  return ret;
}

void DrawField() {
  //Draw playing field
  acquire_screen();
  clear(screen);
  line(screen, 0, 0, scrx-1, 0, makecol(128, 128, 128));
  line(screen, 0, 0, 0, scry-1, makecol(128, 128, 128));
  line(screen, scrx-1, 0, scrx-1, scry-1, makecol(128, 128, 128));
  line(screen, 0, scry-1, scrx-1, scry-1, makecol(128, 128, 128));
  release_screen();
}

void InitPlayers(LinkList<Player>& players) {
  //Init Players reseting them and placing them in a random spot
  players.SetToStart();
  for (int c=0; c<players.Length(); c++) {
    Player* temp = players._GetNext();
    //getrnd(4), the direction, uses the fact that the consts are 0-4 right now
    temp->SetPos(getrnd(scrx-50, 100), getrnd(scry-50, 50), getrnd(4));
    temp->Reset();
    players._Advance();
  }
}

void DoCraters() {
  craterdelay--;
  if (craterdelay < 0) {
    //create a crater
    PlaySound(explosion);
    int x = getrnd(scrx-50, 50);
    int y = getrnd(scry-50, 50);
    int rad = getrnd(scrx/50, 5);
    circlefill(screen, x, y, rad, makecol(getrnd(255), getrnd(255), getrnd(255)));
    circlefill(screen, x, y, rad * 0.75, 0);

    //Reset delay, plus or minus 50 percent
    craterdelay = getrndf(1.5, 0.5) * crater_rate;
  }
}

void PrintResults(LinkList<Player>& players, int dead) {
  //Print out results
  if ((players.Length() - dead) == 1) {
    acquire_screen();
    players.SetToStart();
    Player* winner = NULL;
    int l = players.Length();
    for (int c=0; c<l; c++) {
      winner = players._GetNext();
      if (!winner->IsDead())
        c = l;
      players._Advance();
    }
    textprintf_centre(screen, font, scrx/2, scry/2, makecol(255, 255, 255), "%s is Such a Bastard!", winner->GetName());
    textprintf_centre(screen, font, scrx/2, scry/2+8, makecol(255, 255, 255), "Press ENTER/trigger to continue or ESC");
    release_screen();
    winner->Win(); //Tell this lucky winner he was WON the JACKPOT!
    PlaySound(lose);
  } else if ((players.Length() - dead) == 0) {
    acquire_screen();
    textprintf_centre(screen, font, scrx/2, scry/2, makecol(255, 255, 255), "There was an EXACT tie!  EVERYBODY LOSES! HAHAHAHA!");
    textprintf_centre(screen, font, scrx/2, scry/2+8, makecol(255, 255, 255), "Press ENTER/trigger to continue or ESC");
    release_screen();
    PlaySound(tie);
  }
}